@@ -1,3 +1,4 @@
+# coding=utf-8
 ##############################################################################
 #
 # Copyright (c) 2003 Zope Foundation and Contributors.
@@ -14,24 +15,26 @@
 """Classes which abstract different channels a message could be sent to.
 """
 __docformat__ = 'restructuredtext'
-import socket
+
+from threading import local
+from ssl import SSLError
 from smtplib import SMTP
 
 from zope.interface import implementer
 from zope.sendmail.interfaces import ISMTPMailer
+from zope.sendmail._compat import PY2
 from zope.sendmail._compat import text_type
 
-try:
-    from socket import sslerror as SSLError
-except ImportError:
-    # Py3: The error location changed.
-    from ssl import SSLError
 
-have_ssl = hasattr(socket, 'ssl')
+class _SMTPState(local):
+    connection = None
+    code = None
+    response = None
+
 
 @implementer(ISMTPMailer)
 class SMTPMailer(object):
-
+    """Implementation of :class:`zope.sendmail.interfaces.ISMTPMailer`."""
 
     smtp = SMTP
 
@@ -43,7 +46,17 @@ class SMTPMailer(object):
         self.password = password
         self.force_tls = force_tls
         self.no_tls = no_tls
-        self.connection = None
+        self._smtp = _SMTPState()
+
+    def _make_property(name):
+        return property(lambda self: getattr(self._smtp, name),
+                        lambda self, nv: setattr(self._smtp, name, nv))
+
+    connection = _make_property('connection')
+    code = _make_property('code')
+    response = _make_property('response')
+
+    del _make_property
 
     def vote(self, fromaddr, toaddrs, message):
         self.connection = self.smtp(self.hostname, str(self.port))
@@ -57,49 +70,48 @@ class SMTPMailer(object):
 
         self.code, self.response = code, response
 
+    def _close_connection(self):
+        try:
+            self.connection.quit()
+        except SSLError:
+            # something weird happened while quiting
+            self.connection.close()
+        self.connection = None
 
     def abort(self):
         if self.connection is None:
             return
-
-        try:
-            self.connection.quit()
-        except SSLError:
-            #something weird happened while quiting
-            self.connection.close()
+        self._close_connection()
 
     def send(self, fromaddr, toaddrs, message):
-        connection = getattr(self, 'connection', None)
+        connection = self.connection
         if connection is None:
             self.vote(fromaddr, toaddrs, message)
 
-        connection, code, response = self.connection, self.code, self.response
-
+        connection = self.connection
 
         # encryption support
-        have_tls =  connection.has_extn('starttls')
+        have_tls = connection.has_extn('starttls')
         if not have_tls and self.force_tls:
             raise RuntimeError('TLS is not available but TLS is required')
 
-        if have_tls and have_ssl and not self.no_tls:
+        if have_tls and not self.no_tls:
             connection.starttls()
             connection.ehlo()
 
         if connection.does_esmtp:
             if self.username is not None and self.password is not None:
                 username, password = self.username, self.password
-                if isinstance(username, text_type):
-                    username = username.encode('utf-8')
-                if isinstance(password, text_type):
-                    password = password.encode('utf-8')
+                if PY2 and isinstance(username, text_type):
+                    username = username.encode('utf-8')  # pragma: PY2
+                if PY2 and isinstance(password, text_type):
+                    password = password.encode('utf-8')  # pragma: PY2
                 connection.login(username, password)
         elif self.username:
-            raise RuntimeError('Mailhost does not support ESMTP but a username '
-                                'is configured')
+            raise RuntimeError(
+                'Mailhost does not support ESMTP but a username is configured')
 
-        connection.sendmail(fromaddr, toaddrs, message)
         try:
-            connection.quit()
-        except SSLError:
-            #something weird happened while quiting
-            connection.close()
+            connection.sendmail(fromaddr, toaddrs, message)
+        finally:
+            self._close_connection()
