@@ -139,7 +139,7 @@ class TestQueueProcessorThread(unittest.TestCase):
         self.assertTrue(os.path.exists(md.path))
 
     def test_threadName(self):
-        self.assertEqual(self.thread.getName(),
+        self.assertEqual(self.thread.name,
                          "zope.sendmail.queue.QueueProcessorThread")
 
     def test_parseMessage(self):
@@ -153,6 +153,17 @@ class TestQueueProcessorThread(unittest.TestCase):
         self.assertEqual(t, ('bar@example.com', 'baz@example.com'))
         self.assertEqual(m, msg)
 
+    def test_parseMessage_empty_address(self):
+        hdr = (b'X-Zope-From: foo@example.com\n'
+               b'X-Zope-To: bar@example.com, , baz@example.com\n')
+        msg = (b'Header: value\n'
+               b'\n'
+               b'Body\n')
+        f, t, m = self.thread._parseMessage(hdr + msg)
+        self.assertEqual(f, 'foo@example.com')
+        self.assertEqual(t, ('bar@example.com', 'baz@example.com'))
+        self.assertEqual(m, msg)
+
     def test_parseMessage_error(self):
         msg = b"bad message"
         f, t, m = self.thread._parseMessage(msg)
